Package: Sequential
Type: Package
Title: Exact Sequential Analysis for Poisson and Binomial Data
Version: 4.6.0
Date: 2026-04-20
Authors@R: c(person(given=c("Ivair", "Ramos"),
             family= "Silva", role=c("aut","cre"),
             email= "ivair@ufop.edu.br"),
             person(given= "Martin", family="Kulldorff",role="aut")
             )
Maintainer: Ivair Ramos Silva <ivair@ufop.edu.br>
Description: Functions to calculate exact critical values, statistical power, expected time to signal, and required sample sizes for performing exact sequential analysis. All these	calculations can be done for either Poisson or binomial data, for continuous or group sequential analyses, and for different types of rejection boundaries. In case of group sequential analyses, the group sizes do not have to be specified in advance and the alpha spending can be arbitrarily settled. For regression versions of the methods, Monte Carlo and asymptotic methods are used. 
License: GPL-2
LazyLoad: yes
Imports: boot, pmultinom, maxLik
NeedsCompilation: no
Packaged: 2026-04-20 20:06:33 UTC; User
Author: Ivair Ramos Silva [aut, cre],
  Martin Kulldorff [aut]
Repository: CRAN
Date/Publication: 2026-04-21 05:10:40 UTC
Built: R 4.6.0; ; 2026-04-27 12:44:44 UTC; unix
