exportPattern("^[[:alpha:]]+")
importFrom("graphics", "axis", "legend", "lines", "par", "plot","points")
importFrom("stats", "dbinom", "dpois", "pbinom", "ppois", "qbinom","qexp", "qpois","pnorm", "qnorm")
importFrom("utils", "read.table", "write.table")
importFrom("stats", "aggregate","pnbinom", "rmultinom")
importFrom("boot", "simplex")
importFrom("pmultinom","pmultinom")
importFrom("stats", "rbinom", "rexp", "rnorm")
importFrom("maxLik", "maxLik")
importFrom("stats", "confint")